package be.iminds.ilabt.jfed.experimenter_gui.dialogs;

import be.iminds.ilabt.jfed.experimenter_gui.config.JFedGuiConfig;
import be.iminds.ilabt.jfed.experimenter_gui.events.OpenESpecEvent;
import be.iminds.ilabt.jfed.experimenter_gui.ext_requests.JFedEnvironment;
import be.iminds.ilabt.jfed.git.GitFetcherCommand;
import be.iminds.ilabt.jfed.git.GitFetcherCommandBuilder;
import be.iminds.ilabt.jfed.highlevel.model.SfaModel;
import be.iminds.ilabt.jfed.highlevel.tasks.HighLevelTaskFactory;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.preferences.GuiPreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.*;
import javafx.scene.layout.GridPane;
import javafx.stage.DirectoryChooser;
import javafx.stage.FileChooser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.inject.Inject;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Optional;

/**
 * User: twalcari
 * Date: 2/24/14
 * Time: 4:03 PM
 */
public class OpenESpecController extends GridPane {
    private static final Logger LOG = LoggerFactory.getLogger(OpenESpecController.class);

    private final HighLevelTaskFactory hltf;
    private final GeniUserProvider geniUserProvider;
    private final SfaModel sfaModel;
    private final JFedEnvironment environment;
    private final JFedGuiConfig config;
    private final JFedGuiPreferences preferences;
    private final JFedGuiConfig guiConfig;

    private EventHandler<ActionEvent> actionEventHandler;

    private BooleanProperty valid = new SimpleBooleanProperty(false);


    @FXML
    protected RadioButton archiveSourceButton;
    @FXML
    protected RadioButton dirSourceButton;
    @FXML
    protected RadioButton urlSourceButton;
    @FXML
    protected RadioButton gitSourceButton;

    @FXML
    protected TextField openArchiveTextField;
    @FXML
    protected TextField openDirTextField;
    @FXML
    protected TextField openURLTextField;
    @FXML
    protected TextField openGitRepoTextField;
    @FXML
    protected TextField openGitDirTextField;
    @FXML
    protected TextField openGitBranchTextField;


    @FXML
    protected CheckBox useGitDirCheckBox;
    @FXML
    protected CheckBox useGitBranchCheckBox;


    @Inject
    OpenESpecController(HighLevelTaskFactory hltf, GeniUserProvider geniUserProvider,
                        SfaModel sfaModel, JFedEnvironment environment,
                        JFedGuiConfig config, JFedGuiPreferences preferences,
                        JFedGuiConfig guiConfig) {
        this.hltf = hltf;
        this.geniUserProvider = geniUserProvider;
        this.sfaModel = sfaModel;
        this.environment = environment;
        this.config = config;
        this.preferences = preferences;
        this.guiConfig = guiConfig;
    }


    @FXML
    public void initialize() {
        EventHandler<ActionEvent> eventPasser =
                event -> {
                    if (actionEventHandler != null) actionEventHandler.handle(event);
                };
        openArchiveTextField.setOnAction(eventPasser);
        openDirTextField.setOnAction(eventPasser);
        openURLTextField.setOnAction(eventPasser);
        openGitRepoTextField.setOnAction(eventPasser);

        String initialEspec = preferences.getString(GuiPreferenceKey.PREF_LAST_OPENED_ESPEC);
        if (initialEspec == null)
            initialEspec = guiConfig.getStringFlag("defaultESpecUrl");
        if (initialEspec != null && !initialEspec.trim().isEmpty()) {
            if (initialEspec.startsWith("git ") || initialEspec.startsWith("github ")) {
                gitSourceButton.setSelected(true);
                Optional<GitFetcherCommandBuilder> help = GitFetcherCommandBuilder.fromString(initialEspec);
                if (help.isPresent()) {
                    openGitRepoTextField.setText(help.get().getGitUrl());

                    if (help.get().getBranch() != null && !help.get().getBranch().trim().isEmpty()) {
                        useGitBranchCheckBox.setSelected(true);
                        openGitBranchTextField.setText(help.get().getBranch());
                    } else {
                        useGitBranchCheckBox.setSelected(false);
                    }

                    if (help.get().getRepoSubDir() != null && !help.get().getRepoSubDir().trim().isEmpty()) {
                        useGitDirCheckBox.setSelected(true);
                        openGitDirTextField.setText(help.get().getRepoSubDir());
                    } else {
                        useGitDirCheckBox.setSelected(false);
                    }
                }
            } else {
                if (initialEspec.startsWith("http")) {
                    urlSourceButton.setSelected(true);
                    openURLTextField.setText(initialEspec);
                } else {
                    if (initialEspec.endsWith("/") || new File(initialEspec).isDirectory()) {
                        dirSourceButton.setSelected(true);
                        openDirTextField.setText(initialEspec);
                    } else {
                        archiveSourceButton.setSelected(true);
                        openArchiveTextField.setText(initialEspec);
                    }
                }
            }

            updateValid();
        }
    }


    public void openArchiveAction() {
        FileChooser fc = new FileChooser();
        fc.setTitle("Open Experiment Specification");

        //set the default open dir
        String initialDir = preferences.getString(GuiPreferenceKey.PREF_ESPEC_FILE_DIR);
        File initialDirFile = null;
        if (initialDir != null) {
            initialDirFile = new File(initialDir);

            while (!initialDirFile.exists() && initialDirFile.getParentFile() != null) {
                initialDirFile = initialDirFile.getParentFile();
            }
        } else if (System.getProperty("user.home") != null) {
            initialDirFile = new File(System.getProperty("user.home"));
        }

        if (initialDirFile != null && initialDirFile.exists()) {
            fc.setInitialDirectory(initialDirFile);
        }


        File file = fc.showOpenDialog(openArchiveTextField.getScene().getWindow());

        if (file != null) {
            openArchiveTextField.setText(file.getPath());
            preferences.setString(GuiPreferenceKey.PREF_ESPEC_FILE_DIR, file.getPath());
        }
    }

    public void openDirAction() {
        DirectoryChooser dc = new DirectoryChooser();
        dc.setTitle("Open Experiment Specification");

        //set the default open dir
        String initialDir = preferences.getString(GuiPreferenceKey.PREF_ESPEC_DIR_DIR);
        File initialDirFile = null;
        if (initialDir != null) {
            initialDirFile = new File(initialDir);

            while (!initialDirFile.exists() && initialDirFile.getParentFile() != null) {
                initialDirFile = initialDirFile.getParentFile();
            }
        } else if (System.getProperty("user.home") != null) {
            initialDirFile = new File(System.getProperty("user.home"));
        }

        if (initialDirFile != null && initialDirFile.exists()) {
            dc.setInitialDirectory(initialDirFile);
        }

        File file = dc.showDialog(openDirTextField.getScene().getWindow());

        if (file != null) {
            openDirTextField.setText(file.getPath());
            preferences.setString(GuiPreferenceKey.PREF_ESPEC_DIR_DIR, file.getPath());
        }
    }

    public OpenESpecEvent.ESpecSource getESpecSource() {
        if (archiveSourceButton.isSelected()) return OpenESpecEvent.ESpecSource.ARCHIVE;
        if (dirSourceButton.isSelected()) return OpenESpecEvent.ESpecSource.DIR;
        if (urlSourceButton.isSelected()) return OpenESpecEvent.ESpecSource.URL;
        if (gitSourceButton.isSelected()) return OpenESpecEvent.ESpecSource.GIT;
        throw new IllegalStateException("Radiobuttons are not in an allowed state!");
    }

    public void updateValid() {
        valid.setValue(calcValid());
    }

    public boolean calcValid() {
        switch (getESpecSource()) {
            case ARCHIVE: {
                if (openArchiveTextField.getText() == null ||
                        openArchiveTextField.getText().trim().isEmpty())
                    return false;
                return new File(openArchiveTextField.getText()).exists();
            }
            case DIR: {
                if (openDirTextField.getText() == null ||
                        openDirTextField.getText().trim().isEmpty())
                    return false;
                return new File(openDirTextField.getText()).isDirectory();
            }
            case URL: {
                if (openURLTextField.getText() == null ||
                        openURLTextField.getText().trim().isEmpty())
                    return false;
                try {
                    new URL(openURLTextField.getText());
                    return true;
                } catch (MalformedURLException e) {
                    return false;
                }
            }
            case GIT: {
                if (openGitRepoTextField.getText() == null ||
                        openGitRepoTextField.getText().trim().isEmpty())
                    return false;
                if (useGitBranchCheckBox.isSelected() &&
                        (openGitBranchTextField.getText() == null ||
                                openGitBranchTextField.getText().trim().isEmpty()))
                    return false;
                if (useGitDirCheckBox.isSelected() &&
                        (openGitDirTextField.getText() == null ||
                                openGitDirTextField.getText().trim().isEmpty()))
                    return false;
                return true;
            }
            default: {
                return false;
            }
        }
    }


    public void setOnAction(EventHandler<ActionEvent> actionEventHandler) {
        this.actionEventHandler = actionEventHandler;
    }

    public boolean isValid() {
        return valid.get();
    }

    public BooleanProperty validProperty() {
        return valid;
    }

    @Nonnull
    public String getESpecSourceString() {
        switch (getESpecSource()) {
            case ARCHIVE: {
                return openArchiveTextField.getText();
            }
            case DIR: {
                return openDirTextField.getText();
            }
            case URL: {
                return openURLTextField.getText();
            }
            case GIT: {
                String res = "git ";

                if (openGitRepoTextField.getText().contains("github"))
                    res = "github ";

                res += openGitRepoTextField.getText();

                //syntax:
                //git[hub] https://<githubTLD>/<org>/<repo>.git [<subdir> [<branch>]]

                if (useGitBranchCheckBox.isSelected() || useGitDirCheckBox.isSelected()) {
                    if (useGitDirCheckBox.isSelected())
                        res += " " + openGitDirTextField.getText();
                    else
                        res += " /";
                }

                if (useGitBranchCheckBox.isSelected()) {
                    res += " " + openGitBranchTextField.getText();
                }

                return res;
            }
            default: {
                throw new IllegalStateException("Unhandled ESpec Source: " + getESpecSource());
            }
        }


    }
}
