package be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model;

import be.iminds.ilabt.jfed.espec.util.LimitedLiveLog;
import be.iminds.ilabt.jfed.espec.util.UploadProgressTracker;
import be.iminds.ilabt.jfed.experimenter_gui.ui.status.TaskStatusIndicator;
import javafx.beans.property.*;
import javafx.event.Event;
import javafx.event.EventHandler;
import org.jetbrains.annotations.NotNull;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Objects;


public class ESpecLogItem implements Comparable<ESpecLogItem> {
    /**
     * ID used to identify ESpecLogItems
     */
    private final long id;

    private final ObjectProperty<TaskStatusIndicator.Status> status = new SimpleObjectProperty<>();
    private final StringProperty text = new SimpleStringProperty();
    private final StringProperty statusText = new SimpleStringProperty();
    private final ObjectProperty<EventHandler<Event>> onDetailsView = new SimpleObjectProperty<>();
    private final ObjectProperty<ESpecStateItem> eSpecStateItem = new SimpleObjectProperty<>();

    @Nullable private final UploadProgressTracker uploadProgressTracker;
    @Nullable private final LimitedLiveLog limitedLiveLog;

    private final IntegerProperty logByteSize = new SimpleIntegerProperty();
    private final DoubleProperty progress = new SimpleDoubleProperty();

    public ESpecLogItem(long id, String text, TaskStatusIndicator.Status status,
                        @Nullable UploadProgressTracker uploadProgressTracker,
                        @Nullable LimitedLiveLog limitedLiveLog) {
        this.id = id;
        this.text.set(text);
        this.status.set(status);
        this.uploadProgressTracker = uploadProgressTracker;
        this.limitedLiveLog = limitedLiveLog;

        if (uploadProgressTracker != null)
            uploadProgressTracker.addChangeListener((t) -> progress.setValue(t.getPercentProgress()));

        if (limitedLiveLog != null)
            limitedLiveLog.addChangeListener(new LimitedLiveLog.ChangeListener() {
                @Override
                public void onChanged(@Nonnull LimitedLiveLog source, @Nonnull String addedText) {
                    logByteSize.setValue(source.getReceivedSize());
                }

                @Override
                public void onEndReached(@Nonnull LimitedLiveLog source) {
                    logByteSize.setValue(source.getReceivedSize());
                }
            });
    }

    public ESpecLogItem(long id, String text, TaskStatusIndicator.Status status) {
        this(id, text, status, null, null);
    }

    public ESpecLogItem(long id, String text) {
        this(id, text, TaskStatusIndicator.Status.INACTIVE, null, null);
    }

    public ESpecLogItem(long id) {
        this(id, "");
    }

    public void update(@Nonnull ESpecLogItem eSpecLogItem) {
        this.status.setValue(eSpecLogItem.getStatus());
        this.text.setValue(eSpecLogItem.getText());
        this.statusText.setValue(eSpecLogItem.getStatusText());
    }

    public long getId() {
        return id;
    }

    public TaskStatusIndicator.Status getStatus() {
        return status.get();
    }

    public ObjectProperty<TaskStatusIndicator.Status> statusProperty() {
        return status;
    }

    public String getText() {
        return text.get();
    }

    public StringProperty textProperty() {
        return text;
    }

    public void setStatus(TaskStatusIndicator.Status status) {
        this.status.set(status);
    }

    public void setText(String text) {
        this.text.set(text);
    }

    public EventHandler<Event> getOnDetailsView() {
        return onDetailsView.get();
    }

    public ObjectProperty<EventHandler<Event>> onDetailsViewProperty() {
        return onDetailsView;
    }

    public void setOnDetailsView(EventHandler<Event> onDetailsView) {
        this.onDetailsView.set(onDetailsView);
    }

    public String getStatusText() {
        return statusText.get();
    }

    public StringProperty statusTextProperty() {
        return statusText;
    }

    public void setStatusText(String statusText) {
        this.statusText.set(statusText);
    }

    public ESpecStateItem getESpecStateItem() {
        return eSpecStateItem.get();
    }

    public ObjectProperty<ESpecStateItem> eSpecStateItemProperty() {
        return eSpecStateItem;
    }

    public void setESpecStateItem(ESpecStateItem eSpecStateItem) {
        this.eSpecStateItem.set(eSpecStateItem);
    }

    public int getLogByteSize() {
        return logByteSize.get();
    }

    public IntegerProperty logByteSizeProperty() {
        return logByteSize;
    }

    public double getProgress() {
        return progress.get();
    }

    public DoubleProperty progressProperty() {
        return progress;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        ESpecLogItem that = (ESpecLogItem) o;
        return id == that.id;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }

    @Override
    public int compareTo(@NotNull ESpecLogItem o) {
        return Long.compare(id, o.id);
    }
}
